# -*- coding: utf-8 -*-


from django.utils.translation import ugettext as _
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView, DestroyAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from django.shortcuts import get_list_or_404, get_object_or_404
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST, HTTP_401_UNAUTHORIZED
from rest_framework.views import APIView
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError


from .serializers import BaseReviewListPolymorphicSerializer, BaseReviewDetailsPolymorphicSerializer, \
    LikeListSerializer, LikeDetailsSerializer, ReviewCreateSerializer
from ..models import Review, Like, BaseReview


class BaseReviewStatusSetAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, review_id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user

        content = {
            'is_published': False,
        }

        try:
            obj = get_object_or_404(BaseReview.objects.all(), id=review_id)
            obj.is_published = not obj.is_published
            obj.save()
            content['is_published'] = obj.is_published
            status = HTTP_200_OK

        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        return Response(content, status=status)


class ReviewListAPIView(ListAPIView):
    serializer_class = BaseReviewListPolymorphicSerializer
    permission_classes = [AllowAny]
    # queryset = Review.objects.all()
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)

    def get_queryset(self):
        user = self.request.user
        queryset = Review.objects.all().filter(parent_obj__isnull=True)

        if 'model_id' in self.request.parser_context['kwargs']:
            # Review for model
            dict = {
                'model_obj': self.request.parser_context['kwargs']['model_id'],
                'user_obj': None
            }
            if user.is_authenticated:
                dict['user_obj'] = user
            queryset = Review.objects.model_reviews(**dict)
            return queryset

        if user.is_authenticated:
            # Mine Review
            return Review.objects.get_this_user(user=user).filter(parent_obj__isnull=True)

        return queryset


class BaseReviewChildrenAPIView(ListAPIView):
    serializer_class = BaseReviewListPolymorphicSerializer
    permission_classes = [AllowAny]
    # queryset = Review.objects.all()
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)

    def get_queryset(self):
        user = self.request.user
        queryset = BaseReview.objects.none()
        # all().filter(parent_obj__isnull=True)

        if 'review_id' in self.request.parser_context['kwargs']:
            # Review for model
            dict = {
                'review_id': self.request.parser_context['kwargs']['review_id'],
                'user_obj': None
            }
            if user.is_authenticated:
                dict['user_obj'] = user
            #     TODO: append for admin like review
            queryset = BaseReview.objects.active().filter(parent_obj__pk=dict['review_id'])
            return queryset

        return queryset


class ReviewDetailAPIView(RetrieveAPIView):
    serializer_class = BaseReviewDetailsPolymorphicSerializer
    permission_classes = [AllowAny]
    queryset = Review.objects.all()
    lookup_url_kwarg = 'review_id'
    lookup_field = 'id'


class BaseReviewDeleteAPIView(DestroyAPIView):
    permission_classes = [IsAuthenticated]
    queryset = BaseReview.objects.all()
    lookup_url_kwarg = 'review_id'
    lookup_field = 'id'

    def delete(self, request, *args, **kwargs):
        user = request.user
        model = self.queryset.first().get_real_instance()
        if not user.is_authenticated:
            return Response({"detail": _("Authentication credentials were not provided.")}, status=HTTP_401_UNAUTHORIZED)
        if user.is_admin:
            return self.destroy(request, *args, **kwargs)
        elif model.user_obj == user:
            if BaseReview.objects.filter(parent_obj=model).exists():
                return Response({"detail": _("Authentication credentials were not provided.")}, status=HTTP_401_UNAUTHORIZED)
            return self.destroy(request, *args, **kwargs)
        return Response({"detail": _("Authentication credentials were not provided.")}, status=HTTP_401_UNAUTHORIZED)


class ReviewCreateAPIView(CreateAPIView):
    serializer_class = ReviewCreateSerializer
    permission_classes = [IsAuthenticated]

    def perform_create(self, serializer):
        user = self.request.user
        serializer.save(user_obj=user)


class LikeListAPIView(ListAPIView):
    serializer_class = LikeListSerializer
    permission_classes = [AllowAny]
    # queryset = Like.objects.all()
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)

    def get_queryset(self):
        queryset = Like.objects.all()
        if 'review_id' in self.request.parser_context['kwargs']:
            queryset = Like.objects.like(review_obj=self.request.parser_context['kwargs']['review_id'])
        return queryset


class LikeDetailAPIView(RetrieveAPIView):
    serializer_class = LikeDetailsSerializer
    queryset = Like.objects.all()
    permission_classes = [AllowAny]
    lookup_url_kwarg = 'like_id'
    lookup_field = 'id'


class DislikeListAPIView(ListAPIView):
    serializer_class = LikeListSerializer
    permission_classes = [AllowAny]
    # queryset = Like.objects.all()
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)

    def get_queryset(self):
        queryset = Like.objects.all()
        if 'review_id' in self.request.parser_context['kwargs']:
            queryset = Like.objects.dislike(review_obj=self.request.parser_context['kwargs']['review_id'])
        return queryset


class LikeReviewSetAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, review_id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user

        content = {
            'is_like': False,
        }

        review = get_object_or_404(BaseReview.objects.all(), id=review_id)

        try:

            like = get_object_or_404(Like.objects.review(review_obj=review), user_obj=user)

            if not like.is_like:
                like.is_like = True
                like.save()
                content['is_like'] = True
                status = HTTP_200_OK
                return Response(content, status=status)

            if like.is_like:
                like.delete()

            status = HTTP_200_OK

        except Exception as e:
            try:
                like = Like.objects.create(user_obj=user, review_obj=review, is_like=True)
                like.save()
                content['is_like'] = True
                status = HTTP_200_OK
            except Exception as e:
                pass

        return Response(content, status=status)


class DislikeReviewSetAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, review_id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user

        content = {
            'is_dislike': False,
        }

        review = get_object_or_404(BaseReview.objects.all(), id=review_id)

        try:

            like = get_object_or_404(Like.objects.review(review_obj=review), user_obj=user)

            if like.is_like:
                like.is_like = False
                content['is_dislike'] = True
                like.save()
                status = HTTP_200_OK
                return Response(content, status=status)

            like.delete()
            status = HTTP_200_OK

        except Exception as e:
            try:
                like = Like.objects.create(user_obj=user, review_obj=review, is_like=False)
                like.save()
                content['is_dislike'] = True
                status = HTTP_200_OK
            except Exception as e:
                pass

        return Response(content, status=status)



# -*- coding: utf-8 -*-


from django.utils.translation import ugettext as _
from django.shortcuts import get_object_or_404
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError

from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
from ..models import News


class NewsListAPIView(ListAPIView):
    serializer_class = ModelListPolymorphicSerializer
    permission_classes = [AllowAny]
    queryset = News.objects.active()



# Generated by Django 2.2 on 2021-09-04 11:15

from django.db import migrations, models
import django.db.models.deletion


def add_records(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Course = apps.get_model('courses', 'Course')
    TeacherCourse = apps.get_model('courses', 'TeacherCourse')
    items = [
        TeacherCourse(
            course_obj_id=pk,
            teacher_obj_id=teacher_pk
        )
        for pk, teacher_pk, in Course.objects.filter(teacher_obj__isnull=False).values_list('pk', 'teacher_obj__pk')
    ]

    TeacherCourse.objects.bulk_create(items)


def remove_records(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    TeacherCourse = apps.get_model('courses', 'TeacherCourse')

    TeacherCourse.objects.all().delete()


class Migration(migrations.Migration):
    dependencies = [
        ('teachers', '0003_auto_20181028_2116'),
        ('courses', '0022_coursefilesegment'),
    ]

    operations = [
        migrations.CreateModel(
            name='TeacherCourse',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('priority', models.PositiveIntegerField(default=0, verbose_name='Priority')),
                ('course_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='courses.Course',
                                                 verbose_name='دوره')),
                ('teacher_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='teachers.Teacher',
                                                  verbose_name='مدرس')),
            ],
            options={
                'verbose_name': 'Teacher course',
                'verbose_name_plural': 'Teacher course relations',
            },
        ),
        migrations.AddField(
            model_name='course',
            name='teachers_temporary',
            field=models.ManyToManyField(blank=True, related_name='course_temporary', through='courses.TeacherCourse',
                                         to='teachers.Teacher', verbose_name='مدرسین'),
        ),
        migrations.RunPython(add_records, reverse_code=remove_records),
        migrations.RemoveField(
            model_name='course',
            name='teacher_obj',
        ),
        migrations.RemoveField(
            model_name='course',
            name='teachers_temporary',
        ),
        migrations.AddField(
            model_name='course',
            name='teacher_obj',
            field=models.ManyToManyField(blank=True, through='courses.TeacherCourse', to='teachers.Teacher',
                                         verbose_name='مدرسین'),
        ),
        migrations.AlterModelOptions(
            name='teachercourse',
            options={'ordering': ['priority'], 'verbose_name': 'Teacher course',
                     'verbose_name_plural': 'Teacher course relations'},
        ),
        migrations.AlterUniqueTogether(
            name='teachercourse',
            unique_together={('course_obj', 'teacher_obj')},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-03 02:49


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reviews', '0003_reviewsummary'),
    ]

    operations = [
        migrations.AddField(
            model_name='reviewsummary',
            name='percentage',
            field=models.FloatField(default=10, verbose_name='Percentage'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-03-15 15:02


import aparnik.utils.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bankaccounts', '0002_auto_20190215_1916'),
        ('basemodels', '0010_basemodel_visit_count'),
        ('cosales', '0005_auto_20190315_1347'),
    ]

    operations = [
        migrations.CreateModel(
            name='CoSalePayment',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('price', aparnik.utils.fields.PriceField(decimal_places=0, max_digits=20, verbose_name='Price')),
                ('tracking_number', models.CharField(blank=True, max_length=255, null=True, verbose_name='Tracking Number')),
                ('status', models.CharField(choices=[('CL', 'Cleared'), ('RSHBR', 'The request for settlement has been received'), ('C', 'Cancel')], default='RSHBR', max_length=10, verbose_name='Status')),
                ('user_bank_account_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bankaccounts.BankAccount', verbose_name='User Bank Account')),
            ],
            options={
                'verbose_name': 'Co Sale Payment',
                'manager_inheritance_from_future': True,
                'verbose_name_plural': 'Co Sale Payments',
            },
            bases=('basemodels.basemodel',),
        ),
    ]



from django.conf.urls import url
from django.urls import path
from .views import *

app_name = 'chats'

urlpatterns = [
    url(r'^$', ChatSessionListAPIView.as_view(), name='list'),
    url(r'^create/$', ChatSessionCreateAPIView.as_view(), name='create'),
    path('members/<id>/', ChatSessionMemberDetailAPIView.as_view(), name='members-details'),
    path('messages/<id>', ChatSessionMessageDetailAPIView.as_view(), name='messages-details'),
    path('<uri>/members/', ChatSessionMemberListAPIView.as_view(), name='members'),
    path('<uri>/messages/create', ChatSessionMessageCreateAPIView.as_view(), name='messages-create'),
    path('<uri>/messages/', ChatSessionMessageListAPIView.as_view(), name='messages'),
    path('<uri>/mark-as-read/', ChatSessionMarkAsReadAPIView.as_view(), name='mark-as-read'),
    path('<uri>/', ChatSessionDetailAPIView.as_view(), name='details'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-10 10:03


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0004_auto_20181103_2233'),
    ]

    operations = [
        migrations.AddField(
            model_name='basemodel',
            name='is_show_only_for_super_user',
            field=models.BooleanField(default=False, verbose_name='Show only for super user'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 16:02


from django.db import migrations
from aparnik.utils.utils import convert_iran_phone_number_to_world_number


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0006_user_avatar'),
    ]

    def convert_username(apps, schema_editor):
        '''
        We can't import the Post model directly as it may be a newer
        version than this migration expects. We use the historical version.
        '''
        User = apps.get_model('aparnik_users', 'User')
        for user in User.objects.all():
            user.username = convert_iran_phone_number_to_world_number(user.username)
            user.save()

    operations = [
        migrations.RunPython(convert_username),
    ]



# Generated by Django 2.2 on 2019-07-14 16:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0020_auto_20190615_1430'),
    ]

    operations = [
        migrations.AlterField(
            model_name='coursesummary',
            name='type',
            field=models.CharField(choices=[('M', 'فیلم'), ('V', 'صدا'), ('P', 'پی دی اف'), ('I', 'عکس'), ('L', 'لینک')], max_length=1, verbose_name='نوع'),
        ),
    ]



from django.utils.text import Truncator
from django.urls import reverse
from rest_framework import serializers

from aparnik.settings import aparnik_settings
from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer, BaseModelListSerializer, ModelListPolymorphicSerializer
from ..models import News

UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER


class NewsListSerializer(BaseModelListSerializer):

    user = serializers.SerializerMethodField()
    short_content = serializers.SerializerMethodField()
    cover_images = serializers.SerializerMethodField()
    categories = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(NewsListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = News
        fields = BaseModelListSerializer.Meta.fields + [
            'user',
            'title',
            'slug',
            'cover_images',
            'short_content',
            'is_published',
            'publish_date',
            'categories',
        ]

        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + [
            'user',
            'short_content',
            'cover_images',
            'categories',
        ]

    def get_user(self, obj):
        return UserSummeryListSerializer(obj.user_obj, many=False, read_only=True, context=self.context).data

    def get_short_content(self, obj):
        return Truncator(obj.content).words(30)

    def get_cover_images(self, obj):
        return None if not obj.cover_images.all() else ModelListPolymorphicSerializer(obj.cover_images.all(), many=True, read_only=True, context=self.context).data

    def get_categories(self, obj):
        return None if not obj.categories.all() else ModelListPolymorphicSerializer(obj.categories.all(), many=True, read_only=True, context=self.context).data


# News Details Serializer
class NewsDetailSerializer(NewsListSerializer, BaseModelDetailSerializer):

    def __init__(self, *args, **kwargs):
        super(NewsDetailSerializer, self).__init__(*args, **kwargs)

        # if 'context' in kwargs:
        # self.fields['notification_type'] = NotificationTypeDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = News
        fields = NewsListSerializer.Meta.fields + BaseModelDetailSerializer.Meta.fields + [
            'content',
        ]

        read_only_fields = BaseModelDetailSerializer.Meta.read_only_fields + NewsListSerializer.Meta.read_only_fields + [
            'content']



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-03 00:01


from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('basemodels', '0003_basemodel_update_needed'),
        ('files', '0004_file_cover'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Progresses',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('is_published', models.BooleanField(default=True, verbose_name='Published')),
                ('status', models.CharField(choices=[('SN', 'Not started'), ('SS', 'Started'), ('SM', 'Manual complete'), ('SA', 'Auto complete')], default='SN', max_length=2, verbose_name='Status')),
                ('file_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='files.File', verbose_name='File')),
                ('user_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Progress',
                'verbose_name_plural': 'Progresses',
            },
            bases=('basemodels.basemodel',),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.utils.translation import ugettext_lazy as _

# Create your models here.


class ShortBlogManager(models.Manager):
    def get_queryset(self):
        return super(ShortBlogManager, self).get_queryset().order_by('-id')


class ShortBlog(models.Model):
    title = models.CharField(max_length=255, verbose_name=_('Title'))
    content = models.TextField(verbose_name=_('Content'))

    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = ShortBlogManager()

    def __str__(self):
        return self.title

    def __init__(self, *args, **kwargs):
        super(ShortBlog, self).__init__(*args, **kwargs)

    # class Meta:
    #     verbose_name = 'خبر'
    class Meta:
        verbose_name = _('Short Blog')
        verbose_name_plural = _('Short Blogs')




# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


import aparnik.utils.fields
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Coupon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(blank=True, help_text='If blank default no limitation.', max_length=10, null=True, unique=True, verbose_name='Code')),
                ('expire_at', models.DateTimeField(blank=True, help_text='If blank default no limitation.', null=True, verbose_name='Expire at')),
                ('value', aparnik.utils.fields.PriceField(decimal_places=0, max_digits=8, validators=[django.core.validators.MinValueValidator(1)], verbose_name='Value')),
                ('type', models.CharField(choices=[('p', 'Percent'), ('v', 'Value')], default='p', max_length=2, verbose_name='type')),
                ('max_price_order', aparnik.utils.fields.PriceField(decimal_places=0, default=0, help_text='If blank default no limitation.', max_digits=20, verbose_name='Max price of order')),
                ('min_price_order', aparnik.utils.fields.PriceField(decimal_places=0, default=0, help_text='If blank default no limitation.', max_digits=20, verbose_name='Min price of order')),
                ('for_users', models.CharField(choices=[('a', 'All user'), ('l', 'Limited user'), ('c', 'Custom user')], default='a', max_length=2, verbose_name='For users')),
                ('for_limited_users_value', models.IntegerField(blank=True, null=True, verbose_name='Limited users value')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
            ],
            options={
                'ordering': ('-created_at',),
                'verbose_name': 'Coupon',
                'verbose_name_plural': 'Coupons',
            },
        ),
        migrations.CreateModel(
            name='Membership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_redeem', models.BooleanField(default=False, verbose_name='Is redeem')),
                ('coupon', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='coupons.Coupon', verbose_name='Coupon')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='coupons', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Membership',
                'verbose_name_plural': 'Memberships',
            },
        ),
        migrations.AddField(
            model_name='coupon',
            name='users',
            field=models.ManyToManyField(through='coupons.Membership', to=settings.AUTH_USER_MODEL, verbose_name='Membership'),
        ),
        migrations.CreateModel(
            name='CouponForAllUser',
            fields=[
            ],
            options={
                'verbose_name': 'Coupon for all users',
                'proxy': True,
                'verbose_name_plural': 'Coupons for all users',
                'indexes': [],
            },
            bases=('coupons.coupon',),
        ),
        migrations.CreateModel(
            name='CouponForCoustomUser',
            fields=[
            ],
            options={
                'verbose_name': 'Coupon for custom users',
                'proxy': True,
                'verbose_name_plural': 'Coupons for custom users',
                'indexes': [],
            },
            bases=('coupons.coupon',),
        ),
        migrations.CreateModel(
            name='CouponForLimitedUser',
            fields=[
            ],
            options={
                'verbose_name': 'Coupon for limited users',
                'proxy': True,
                'verbose_name_plural': 'Coupons for limited users',
                'indexes': [],
            },
            bases=('coupons.coupon',),
        ),
    ]



# Generated by Django 2.2 on 2020-06-02 09:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0014_auto_20200210_1451'),
    ]

    operations = [
        migrations.AlterField(
            model_name='filefield',
            name='password',
            field=models.CharField(blank=True, default='', max_length=64, null=True, verbose_name='گذرواژه'),
        ),
    ]



# -*- coding: utf-8 -*-

import datetime
from django.contrib import admin
from django.utils.html import format_html
from import_export import resources, fields
from import_export.admin import ExportActionMixin
from rangefilter.filter import DateRangeFilter, DateTimeRangeFilter
from aparnik.contrib.users.admin import get_update_at, get_user_search_fields
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelTabularInline
from .models import Order, OrderItem
from django.utils.translation import gettext_lazy as _

# Rcegister your models here.
from ..payments.models import Payment


class OrderItemInline(BaseModelTabularInline):
    model = OrderItem
    raw_id_fields = ['product_obj']
    fk_name = 'order_obj'
    exclude = []
    extra = 1

    def __init__(self, *args, **kwargs):
        Klass = OrderItemInline
        Klass_parent = BaseModelTabularInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude


# class OrderItemAdmin(BaseModelAdmin):
#     list_display = ['order_obj', 'product', 'price', 'quantity']
#     raw_id_fields = ['order_obj']
#     search_fields = ['order_obj__id', 'product_obj__title']
#     fields = ['__all__']
#     inlines = [OrderItemInline]
#     list_filter = ['status', 'created_at']
#     exclude = []
#     dynamic_raw_id_fields = []
#
#     def __init__(self, *args, **kwargs):
#         Klass = OrderItemAdmin
#         Klass_parent = BaseModelAdmin
#
#         super(Klass, self).__init__(*args, **kwargs)
#         self.fields = Klass_parent.fields + self.fields
#         self.list_display = Klass_parent.list_display + self.list_display
#         self.list_filter = Klass_parent.list_filter + self.list_filter
#         self.search_fields = Klass_parent.search_fields + self.search_fields
#         self.exclude = Klass_parent.exclude + self.exclude
#         self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
#         self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields
#
#     class Meta:
#         OrderItem

def invoice_link(obj):
    return format_html(
        '<a href="{0}" target="blank" class="button">Invoice</a>',
        obj.get_pay_uri()
    )


invoice_link.short_description = "Invoice"


class PaymentInline(admin.TabularInline):
    model = Payment
    fk_name = 'order_obj'
    show_change_link = True
    extra = 0

    def get_readonly_fields(self, request, obj=None):
        return list(set(
            [field.name for field in self.opts.local_fields] +
            [field.name for field in self.opts.local_many_to_many]
        ))

    def has_change_permission(self, request, obj=None):
        return False


class OrderResource(resources.ModelResource):
    order_code = fields.Field(attribute='order_code', column_name=_('Order code'))
    user = fields.Field(attribute='user_id', column_name=_('User ID'))
    uuid = fields.Field(attribute='uuid', column_name=_('UUID'))
    total_cost_order = fields.Field(attribute='total_cost_order', column_name=_('Total cost'))
    discount_cost = fields.Field(attribute='discount_cost', column_name=_('Discount cost'))
    total_cost = fields.Field(attribute='total_cost', column_name=_('Total payment'))
    coupon = fields.Field(attribute='coupon__code', column_name=_('Coupon code'))
    status = fields.Field(attribute='status', column_name=_('Status payment'))
    created_at = fields.Field(attribute='created_at', column_name=_('Created date'))
    update_at = fields.Field(attribute='update_at', column_name=_('Update date'))
    payment_link = fields.Field(attribute='payment_link', column_name=_('Invoice link'))

    class Meta:
        model = Order
        fields = ['user', 'uuid', 'status', 'created_at', 'update_at', 'tax_cost']
        exclud = ['address_obj', 'postal_cost_value', 'is_sync_with_websites']

    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(OrderResource, self).__init__()

    def dehydrate_order_code(self, obj):
        return obj.order_code

    def dehydrate_total_cost_order(self, obj):
        return "%s" % obj.get_total_cost_order()

    def dehydrate_total_cost(self, obj):
        return "%s" % obj.get_total_cost()

    def dehydrate_discount_cost(self, obj):
        return '%s' % obj.get_discount()

    def dehydrate_payment_link(self, obj):
        return "%s" % self.request.build_absolute_uri(obj.get_pay_uri())

    # Optimization of exporting operations with related models
    def get_queryset(self, queryset=None):
        if queryset is None:
            queryset = Order.objects.all()
        return queryset.prefetch_related('coupon', 'items')

    def export(self, queryset=None):
        queryset = self.get_queryset(queryset)
        fetched_queryset= list(queryset)
        return super().export(fetched_queryset)


class OrderAdmin(ExportActionMixin, BaseModelAdmin):
    resource_class = OrderResource
    # list_display = ['id', 'first_name', 'last_name', 'email',
    #                 'address', 'postal_code', 'city', 'paid',
    #                 'created', 'updated']
    # list_filter = ['paid', 'created', 'updated']
    fields = ['user', 'status', 'address_obj', 'whom', ]
    list_display = ['user', 'coupon', 'uuid', invoice_link, 'status', 'whom', 'is_sync_with_websites', get_update_at]
    inlines = [OrderItemInline, PaymentInline]
    list_filter = ['status', 'whom', 'is_sync_with_websites', ('created_at', DateRangeFilter), ('update_at', DateTimeRangeFilter)]
    search_fields = get_user_search_fields('user') + ['coupon__code', 'uuid', 'items__product_obj__title']
    exclude = []
    raw_id_fields = ['user', 'coupon', 'address_obj']
    dynamic_raw_id_fields = []

    def get_resource_kwargs(self, request, *args, **kwargs):
        return {'request': request}

    def __init__(self, *args, **kwargs):
        Klass = OrderAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields


admin.site.register(Order, OrderAdmin)
# admin.site.register(OrderItem, OrderItemAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-13 16:06


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0008_auto_20181211_1552'),
        ('segments', '0002_auto_20181026_1745'),
    ]

    operations = [
        migrations.CreateModel(
            name='SegmentSort',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.IntegerField(default=0, verbose_name='\\u0645\\u0631\\u062a\\u0628 \\u0633\\u0627\\u0632\\u06cc')),
                ('model_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='segment_sort_model', to='basemodels.BaseModel', verbose_name='\\u0686\\u0647 \\u0686\\u06cc\\u0632 \\u0631\\u0627 \\u0646\\u0645\\u0627\\u06cc\\u0634 \\u062f\\u0647\\u062f\\u061f')),
                ('segment_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='segment_sort_segment', to='segments.BaseSegment', verbose_name='Segment')),
            ],
            options={
                'verbose_name': 'Segment Sort',
                'verbose_name_plural': 'Segments Sort',
            },
        ),
    ]



from django.conf.urls import url, include

from .views import ContactUsListAPIView, ContactUsDetailAPIView, ContactUsCreateAPIView

app_name = 'contactus'

urlpatterns = [
    url(r'^$', ContactUsListAPIView.as_view(), name='list'),
    url(r'^(?P<id>\d+)/$', ContactUsDetailAPIView.as_view(), name='details'),
    url(r'^create/$', ContactUsCreateAPIView.as_view(), name='create'),
]



from django.conf.urls import url, include

app_name = 'shops'

urlpatterns = [
    url(r'^products/', include('aparnik.packages.shops.products.api.urls', namespace='products')),
    url(r'^payments/', include('aparnik.packages.shops.payments.api.urls', namespace='payments')),
    url(r'^orders/', include('aparnik.packages.shops.orders.api.urls', namespace='orders')),
    url(r'^files/', include('aparnik.packages.shops.files.api.urls', namespace='files')),
    url(r'^productssharing/', include('aparnik.packages.shops.productssharing.api.urls', namespace='productssharing')),
    url(r'^vouchers/', include('aparnik.packages.shops.vouchers.api.urls', namespace='vouchers')),
    url(r'^subscriptions/', include('aparnik.packages.shops.subscriptions.api.urls', namespace='subscriptions')),
    url(r'^cosales/', include('aparnik.packages.shops.cosales.api.urls', namespace='cosales')),
    url(r'^coupons/', include('aparnik.packages.shops.coupons.api.urls', namespace='coupons')),
]



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.users.admin import get_update_at
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelTabularInline
from aparnik.packages.shops.products.admin import ProductAdmin
from aparnik.packages.shops.files.admin import FileAdmin
from .models import Course, CourseFile, CourseSummary, BaseCourse


class TeacherCourseInline(BaseModelTabularInline):
    model = Course.teacher_obj.through
    fields = ['teacher_obj', 'priority', ]
    raw_id_fields = ('teacher_obj',)
    extra = 1
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = TeacherCourseInline
        Klass_parent = BaseModelTabularInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude



class CourseFileSortInline(admin.TabularInline):
    model = CourseFile
    fk_name = 'course_obj'
    fields = ['title', 'sort']
    extra = 1


class ParentFilter(admin.SimpleListFilter):
    title = _('Parent Filter')
    parameter_name = 'parent_filter'

    def lookups(self, request, model_admin):
        return (
            ('Yes', 'Yes'),
            ('No', 'No'),
        )

    def queryset(self, request, queryset):
        value = self.value()
        if value == 'Yes':
            return queryset.filter(parent_obj__isnull=False)
        elif value == 'No':
            return queryset.filter(parent_obj__isnull=True)
        return queryset


class DependencyFilter(admin.SimpleListFilter):
    title = _('Dependency Filter')
    parameter_name = 'dependency_filter'

    def lookups(self, request, model_admin):
        return (
            ('Yes', 'Yes'),
            ('No', 'No'),
        )

    def queryset(self, request, queryset):
        value = self.value()
        if value == 'Yes':
            return queryset.filter(dependency_obj__isnull=False)
        elif value == 'No':
            return queryset.filter(dependency_obj__isnull=True)
        return queryset


# Register your models here.
class CourseSummaryAdmin(BaseModelAdmin):
    fields = []
    list_display = ['total_time', 'type', 'file_count', 'file_count_preview']
    search_fields = ['total_time', ]
    # raw_id_fields = ['course']
    list_filter = ['type', ]
    exclude = []
    dynamic_raw_id_fields = []
    raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = CourseSummaryAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = CourseSummary


class BaseCourseAdmin(ProductAdmin):

    fields = ['description', ]
    list_display = []
    list_filter = []
    search_fields = []
    exclude = []
    dynamic_raw_id_fields = []
    raw_id_fields = []
    inlines = []

    def __init__(self, *args, **kwargs):
        Klass = BaseCourseAdmin
        Klass_parent = ProductAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields
        self.inlines = Klass_parent.inlines + self.inlines

    class Meta:
        model = BaseCourse


class CourseAdmin(BaseCourseAdmin):
    fields = [
        'banner',
        'cover',
        'category_obj',
        'parent_obj',
        'dependency_obj',
        'publish_date',
        'publish_month',
        'publish_week',
        'publish_day'
    ]
    list_display = [
        'parent_obj',
        'category_obj',
        'price_fabric',
        'dependency_obj',
        'publish_month',
        'publish_week',
        'publish_day',
        get_update_at,
    ]

    search_fields = [
        'parent_obj__title',
        'category_obj__title',
    ]
    list_filter = [
        ParentFilter,
        DependencyFilter,
    ]# 'publish_month', 'publish_week', 'publish_day']
    raw_id_fields = []
    exclude = []
    dynamic_raw_id_fields = [
        'category_obj',
        'dependency_obj',
        'parent_obj',
        'cover',
        'banner',
    ]
    inlines = [
        CourseFileSortInline,
        TeacherCourseInline,
    ]

    def __init__(self, *args, **kwargs):
        Klass = CourseAdmin
        Klass_parent = BaseCourseAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields
        self.inlines = Klass_parent.inlines + self.inlines

    class Meta:
        model = Course

    def save_model(self, request, obj, form, change):
        super(CourseAdmin, self).save_model(request, obj, form, change)
        # must after super because first one parent save and set the default value for permission after that save model
        obj.update_needed = True
        obj.save()
        return obj


class CourseFileAdmin(FileAdmin):
    fields = ['course_obj']
    list_display = ['course_obj', 'price', get_update_at, ]
    search_fields = ['course_obj__title', ]
    list_filter = []
    raw_id_fields = ['course_obj', ]
    exclude = []
    dynamic_raw_id_fields = []        # exclude = ('password', 'iv',)
    inlines = []

    def __init__(self, *args, **kwargs):
        Klass = CourseFileAdmin
        Klass_parent = FileAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields
        self.inlines = Klass_parent.inlines + self.inlines

    class Meta:
        model = CourseFile

    def save_model(self, request, obj, form, change):
        super(CourseFileAdmin, self).save_model(request, obj, form, change)
        # must after super because first one parent save and set the default value for permission after that save model
        obj.update_needed = True
        obj.save()


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.admin import SegmentAdmin
    from .models import CourseSegment, CourseFileSegment

    class CourseSegmentAdmin(SegmentAdmin):
        fields = []
        list_display = []
        search_fields = []
        list_filter = []
        raw_id_fields = []
        exclude = []
        dynamic_raw_id_fields = []

        def __init__(self, *args, **kwargs):
            Klass = CourseSegmentAdmin
            Klass_parent = SegmentAdmin

            super(Klass, self).__init__(*args, **kwargs)
            self.fields = Klass_parent.fields + self.fields
            self.list_display = Klass_parent.list_display + self.list_display
            self.list_filter = Klass_parent.list_filter + self.list_filter
            self.search_fields = Klass_parent.search_fields + self.search_fields
            self.exclude = Klass_parent.exclude + self.exclude
            self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
            self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

        class Meta:
            model = CourseSegment


    class CourseFileSegmentAdmin(SegmentAdmin):
        fields = []
        list_display = []
        search_fields = []
        list_filter = []
        raw_id_fields = []
        exclude = []
        dynamic_raw_id_fields = []

        def __init__(self, *args, **kwargs):
            Klass = CourseFileSegmentAdmin
            Klass_parent = SegmentAdmin

            super(Klass, self).__init__(*args, **kwargs)
            self.fields = Klass_parent.fields + self.fields
            self.list_display = Klass_parent.list_display + self.list_display
            self.list_filter = Klass_parent.list_filter + self.list_filter
            self.search_fields = Klass_parent.search_fields + self.search_fields
            self.exclude = Klass_parent.exclude + self.exclude
            self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
            self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

        class Meta:
            model = CourseFileSegment

    admin.site.register(CourseSegment, CourseSegmentAdmin)
    admin.site.register(CourseFileSegment, CourseFileSegmentAdmin)

admin.site.register(Course, CourseAdmin)
admin.site.register(CourseFile, CourseFileAdmin)



# Generated by Django 2.2 on 2021-12-23 20:12

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0018_auto_20190423_1058'),
        ('coupons', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProductCoupon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('coupon_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='coupons.Coupon', verbose_name='Coupon')),
                ('product_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='products.Product', verbose_name='Product')),
            ],
            options={
                'verbose_name': 'Product Coupon',
                'verbose_name_plural': 'Product Coupon relations',
            },
        ),
        migrations.AddField(
            model_name='coupon',
            name='products',
            field=models.ManyToManyField(blank=True, through='coupons.ProductCoupon', to='products.Product', verbose_name='Products'),
        ),
    ]



# Generated by Django 2.2 on 2019-08-21 15:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0008_auto_20190810_1836'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='description',
            field=models.TextField(verbose_name='Message'),
        ),
    ]



"""Base Implementation of a Delivery Backend."""

import abc
from json import dumps
import six
import channels.layers
from asgiref.sync import async_to_sync

from django.contrib.auth import get_user_model

from . import default_settings as settings

"""import push notification"""
from django.core.management import BaseCommand
from django.core.serializers.json import DjangoJSONEncoder
from django.contrib.auth import get_user_model
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta
from django.utils.text import Truncator


import json

from fcm_django.models import FCMDevice
from aparnik.utils.utils import get_request
from aparnik.contrib.settings.models import Setting
from aparnik.contrib.notifications.models import Notification
from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
from aparnik.settings import aparnik_settings

User = get_user_model()
UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER

"""end import push notification"""
@six.add_metaclass(abc.ABCMeta)
class BaseNotificationChannel():
    """Base channel for sending notifications."""

    def __init__(self, **kwargs):
        self.notification_kwargs = kwargs

    @abc.abstractmethod
    def construct_message(self):
        """Constructs a message from notification details."""
        pass

    @abc.abstractmethod
    def notify(self, message):
        """Sends the notification."""
        pass


class ConsoleChannel(BaseNotificationChannel):
    """Dummy channel that prints to the console."""

    def construct_message(self):
        """Stringify the notification kwargs."""
        return str(self.notification_kwargs)

    def notify(self, message):
        print(message)


class BasicWebSocketChannel(BaseNotificationChannel):
    """It creates a Redis user for each user (based on their username)."""

    def _connect(self):
        channel_layer = channels.layers.get_channel_layer()
        return channel_layer

    def construct_message(self):
        """Construct message from notification details."""

        return self.notification_kwargs

    def notify(self, message):
        """
        Puts a new message on the queue.
        
        The queue is named based on the username (for Uniqueness)
        """
        uri = self.notification_kwargs['uri']

        if not uri:
            return

        channel_layer = self._connect()
        # add this moment work only with user, not guest
        async_to_sync(channel_layer.group_send)(
            uri,
            {
                "type": "send_data",
                'data': message,
            }
        )

    # # Get user instance
    # User = get_user_model()
    # source = User.objects.get(id=message['source'])
    # recipient = User.objects.get(id=message['recipient'])
    #
    # channel.queue_declare(queue=source.username)
    #
    # jsonified_messasge = dumps(message)
    # channel.basic_publish(
    #     exchange='', routing_key=recipient.username,
    #     body=jsonified_messasge
    # )


class BasicPushNotificationChannel(BaseNotificationChannel):

    def construct_message(self):
        """Construct message from notification details."""
        username_source = self.notification_kwargs['source']
        user_source_serial=None
        if username_source:
            user_source = User.objects.filter(username=username_source).first()
            user_source_serial = json.dumps(UserSummeryListSerializer(user_source,
                                                               many=False,
                                                               read_only=True,
                                                               context={'request': get_request()},
                                                               ).data,
                                     sort_keys=True,
                                     indent=1,
                                     cls=DjangoJSONEncoder
                                     ) if user_source else None
        message = {
            'title': self.notification_kwargs['short_description'],
            'body': '',
            'data': {
                'model': self.notification_kwargs['extra_data']['message'],
                'from_user': user_source_serial,
                'type': Notification.NOTIFICATION_INFO,
                'title': self.notification_kwargs['short_description'],
                'description': '',

            }
        }
        return message

    def notify(self, message):
        """
        Puts a new message on the queue.

        The queue is named based on the username (for Uniqueness)
        """
        uri = self.notification_kwargs['uri']
        recipient = self.notification_kwargs['recipient']

        if not uri and not recipient:
            return
        try:
            devices = FCMDevice.objects.filter(user__in=User.objects.filter(username=recipient)).order_by('device_id').distinct()
            sent_result = devices.send_message(
                **message
            )
            print(sent_result)
        except Exception as inst:
            print("Error happen")
            print(inst)



# -*- coding: utf-8 -*-


from django.core.management import BaseCommand
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta

import subprocess
import os

from aparnik.contrib.filefields.models import FileField


class Command(BaseCommand):
    # Show this when the user types help
    help = "Generate multiple qualities File fields"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        queryset = FileField.objects.multi_quality()

        for file in queryset:
            # check file exist
            path = file.file_direct.path
            infile = path
            outfile = '%s%s'%(path, file.file_another_quality)

            if file.type == FileField.FILE_MOVIE:
                file.multi_quality_processing = 1
                if file.is_lock:
                    is_lock = file.is_lock
                    # unlock
                    file.is_lock = not is_lock
                    file.save()
                    file.is_lock = is_lock
                subprocess.call(
                    "ffmpeg -i '%s' -preset slow -codec:v libx264 -vf scale=-1:360 '%s'"%(path, outfile),
                    shell=True, close_fds=True
                )

            file.multi_quality_processing = 2
            if file.is_lock:
                is_lock = file.is_lock
                # unlock
                file.is_lock = not is_lock
                file.save()
                file.is_lock = is_lock
            file.save()

            # Finish and release lock
            # file.is_lock = False
            # file.save()

        finished_time = now()

        print(('multiple qualities done %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))



from django.conf.urls import url, include

from .views import BookListAPIView, BookUserListAPIView

app_name = 'books'

urlpatterns = [
    url(r'^$', BookListAPIView.as_view(), name='list'),
    url(r'^user/$', BookUserListAPIView.as_view(), name='user'),
]



# -*- coding: utf-8 -*-


from django.contrib import admin
from jalali_date.admin import ModelAdminJalaliMixin
from aparnik.utils.utils import is_app_installed
from aparnik.contrib.users.admin import get_update_at
from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.packages.shops.files.admin import FileAdmin
from .models import Book, Publisher, WriterTranslator


# Register your models here.
class WriterTranslatorAdmin(BaseModelAdmin):
    list_display = ['first_name', 'last_name', get_update_at, ]
    search_fields = ['id', 'first_name', 'last_name', ]
    list_filter = ['created_at', ]
    fields = ['first_name', 'last_name']
    exclude = []
    raw_id_fields = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = WriterTranslatorAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = WriterTranslator


class PublisherAdmin(BaseModelAdmin):
    list_display = ['title', get_update_at, ]
    search_fields = ['title', ]
    list_filter = ['created_at', ]
    exclude = []
    raw_id_fields = []
    dynamic_raw_id_fields = []
    fields = ['title']

    def __init__(self, *args, **kwargs):
        Klass = PublisherAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Publisher


class BookAdmin(FileAdmin):
    fields = ['sample_book', 'published_date', 'category_obj', 'publisher_obj', 'writers_obj', 'translators_obj', ]
    list_display = [get_update_at, ]
    search_fields = []
    list_filter = ['created_at', ]
    raw_id_fields = ['category_obj', 'publisher_obj', 'sample_book']
    exclude = []
    dynamic_raw_id_fields = []    

    def __init__(self, *args, **kwargs):
        Klass = BookAdmin
        Klass_parent = FileAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Book


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.admin import SegmentAdmin
    from .models import BookSegment

    class BookSegmentAdmin(SegmentAdmin):
        fields = []
        list_display = []
        search_fields = []
        list_filter = []
        raw_id_fields = []
        exclude = []
        dynamic_raw_id_fields = []

        def __init__(self, *args, **kwargs):
            Klass = BookSegmentAdmin
            Klass_parent = SegmentAdmin

            super(Klass, self).__init__(*args, **kwargs)
            self.fields = Klass_parent.fields + self.fields
            self.list_display = Klass_parent.list_display + self.list_display
            self.list_filter = Klass_parent.list_filter + self.list_filter
            self.search_fields = Klass_parent.search_fields + self.search_fields
            self.exclude = Klass_parent.exclude + self.exclude
            self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
            self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

        class Meta:
            model = BookSegment


    admin.site.register(BookSegment, BookSegmentAdmin)

admin.site.register(Book, BookAdmin)
admin.site.register(WriterTranslator, WriterTranslatorAdmin)
admin.site.register(Publisher, PublisherAdmin)




# -*- coding: utf-8 -*-


from django.contrib import admin
from django.utils.timezone import now
from django.contrib.admin import SimpleListFilter
from django.utils.translation import ugettext_lazy as _
from jalali_date.admin import ModelAdminJalaliMixin, StackedInlineJalaliMixin, TabularInlineJalaliMixin
from aparnik.contrib.users.admin import get_update_at, get_user_search_fields
from .models import CouponForAllUser, CouponForCoustomUser, CouponForLimitedUser, ProductCoupon


class CouponListFilter(SimpleListFilter):
    title = _('Expire Coupon')
    parameter_name = 'expire_at'

    def lookups(self, request, model_admin):
        return (
            ('all', _('All Coupon')),
            (None, _('Available')),
            ('expire', _('Expire Coupons')),
        )

    def queryset(self, request, queryset):
        if self.value() == 'all':
            return queryset.all()
        elif self.value() == None:
            return queryset.filter(expire_at__date__gte=now())
        elif self.value() == 'expire':
            return queryset.filter(expire_at__date__lt=now())

    def choices(self, cl):
        yield {
            'selected': self.value() is None,
            'query_string': cl.get_query_string({}, [self.parameter_name]),
            'display': _('Available'),  # Changed All to Hide archived.
        }
        for lookup, title in self.lookup_choices:
            # TODO delete the default one !
            if str(lookup) == 'None':
                continue
            yield {
                'selected': self.value() == str(lookup),
                'query_string': cl.get_query_string({
                    self.parameter_name: lookup,
                }, []),
                'display': title,
            }


class PaymentCouponInline(admin.TabularInline):
    model = ProductCoupon
    fields = ['product_obj', ]
    raw_id_fields = ['product_obj', ]


# Register your models here.
class CouponAdmin(ModelAdminJalaliMixin,admin.ModelAdmin):
    list_display = (
        'id', 'code', 'value', 'type', 'max_price_order', 'min_price_order', get_update_at, 'expire_at',
    )
    list_filter = [CouponListFilter, 'type']
    inlines = [PaymentCouponInline, ]
    search_fields = ['code', 'type', 'value']
    actions = None

     # permission

    def change_view(self, request, object_id, form_url='', extra_context=None):
        ''' customize edit form '''
        extra_context = extra_context or {}
        extra_context['show_save_and_continue'] = False
        extra_context['show_save'] = False
        extra_context['show_save_and_add_another'] = False  # this not works if has_add_permision is True
        return super(CouponAdmin, self).change_view(request, object_id, extra_context=extra_context)


class CouponForAllUserAdmin(CouponAdmin):

    exclude = ('for_users', 'for_limited_users_value')


class MembershipInline(admin.TabularInline):
    model = CouponForCoustomUser.users.through
    fields = ('user', 'is_redeem')
    readonly_fields = ('is_redeem', )
    raw_id_fields = ('user',)
    extra = 1


class CouponForLimitedUserAdmin(CouponAdmin):
    search_fields = CouponAdmin.search_fields + get_user_search_fields('users')
    exclude = ('for_users', )
    inlines = (MembershipInline,)


class CouponForCoustomUserAdmin(CouponAdmin):
    search_fields = CouponAdmin.search_fields + get_user_search_fields('users')
    exclude = ('for_users', 'for_limited_users_value')
    inlines = (MembershipInline,)


admin.site.register(CouponForAllUser, CouponForAllUserAdmin)
admin.site.register(CouponForCoustomUser, CouponForCoustomUserAdmin)
admin.site.register(CouponForLimitedUser, CouponForLimitedUserAdmin)


from rest_framework.response import Response
from rest_framework.status import HTTP_201_CREATED, HTTP_400_BAD_REQUEST
from rest_framework.generics import (
    CreateAPIView,
    # DestroyAPIView,
    ListAPIView,
    # UpdateAPIView,
    RetrieveAPIView,
    # RetrieveUpdateAPIView
    )
from rest_framework.permissions import (
    AllowAny,
    IsAuthenticated,
    IsAdminUser,
    IsAuthenticatedOrReadOnly,

    )
from rest_framework.pagination import PageNumberPagination
from django.db.models import Q

from aparnik.contrib.province.models import Province, City, Shahrak, Town
from aparnik.contrib.province.api.serializers import ProvinceListSerializer, ProvinceCreateSerializer, ProvinceDetailSerializer, \
    CityListSerializer, CityCreateSerializer, CityDetailSerializer, \
    ShahrakCreateSerializer, ShahrakDetailSerializer, ShahrakListSerializer, \
    TownCreateSerializer, TownDetailSerializer, TownListSerializer

permission_create = [IsAdminUser]
permission_read = [AllowAny]

class LargeResultsSetPagination(PageNumberPagination):
    page_size = 100
    page_size_query_param = 'page_size'
    max_page_size = 10

def get_province(kwargs):
    return kwargs.get('province_id', 0)

def get_city(kwargs):
    return kwargs.get('city_id', 0)

class ProvinceListAPIView(ListAPIView):
    serializer_class = ProvinceListSerializer
    permission_classes = permission_read
    # http_method_names = ['get']
    queryset = Province.objects.all()
    pagination_class = LargeResultsSetPagination

class ProvinceCreateAPIView(CreateAPIView):
    serializer_class = ProvinceCreateSerializer
    permission_classes = permission_create
    #http_method_names = ['get']

class ProvinceDetailAPIView(RetrieveAPIView):
    queryset = Province.objects.all()
    serializer_class = ProvinceDetailSerializer
    permission_classes = permission_read
    lookup_field = 'id'

# City
class CityListAPIView(ListAPIView):
    serializer_class = CityListSerializer
    permission_classes = permission_read
    # http_method_names = ['get']
    pagination_class = LargeResultsSetPagination

    def get_queryset(self):
        """
        This view should return a list of all the purchases
        for the currently authenticated user.
        """
        province = get_province(self.kwargs)
        return City.objects.filter(province=province)

class CityCreateAPIView(CreateAPIView):
    serializer_class = CityCreateSerializer
    permission_classes = permission_create
    #http_method_names = ['get']

class CityDetailAPIView(RetrieveAPIView):
    queryset = City.objects.all()
    serializer_class = CityDetailSerializer
    permission_classes = permission_read
    lookup_field = 'id'

# Shahrak
class ShahrakListAPIView(ListAPIView):
    serializer_class = ShahrakListSerializer
    permission_classes = permission_read
    # http_method_names = ['get']
    pagination_class = LargeResultsSetPagination

    def get_queryset(self):
        """
        This view should return a list of all the purchases
        for the currently authenticated user.
        """
        province = get_province(self.kwargs)
        return Shahrak.objects.filter(province=province)

class ShahrakCreateAPIView(CreateAPIView):
    serializer_class = ShahrakCreateSerializer
    permission_classes = permission_create
    #http_method_names = ['get']

class ShahrakDetailAPIView(RetrieveAPIView):
    queryset = Shahrak.objects.all()
    serializer_class = ShahrakDetailSerializer
    permission_classes = permission_read
    lookup_field = 'id'

# Town
class TownListAPIView(ListAPIView):
    serializer_class = TownListSerializer
    permission_classes = permission_read
    # http_method_names = ['get']
    queryset = Town.objects.all()
    pagination_class = LargeResultsSetPagination

    def get_queryset(self):

        city = get_city(self.kwargs)
        return Town.objects.filter(city=city)

class TownCreateAPIView(CreateAPIView):
    serializer_class = TownCreateSerializer
    permission_classes = permission_create
    #http_method_names = ['get']

class TownDetailAPIView(RetrieveAPIView):
    queryset = Town.objects.all()
    serializer_class = TownDetailSerializer
    permission_classes = permission_read
    lookup_field = 'id'


from django.conf.urls import url, include

from .views import TermsAndConditionsDetailAPIView

app_name = 'termsandconditions'

urlpatterns = [
    url(r'^$', TermsAndConditionsDetailAPIView.as_view(), name='detail'),
]


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-02-15 17:41


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0015_auto_20190214_1441'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='username_mention',
            field=models.CharField(default='a', max_length=100, unique=True, verbose_name='Username for mention'),
        ),
    ]



# -*- coding: utf-8 -*-


from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.settings import Setting
from aparnik.api.serializers import ModelSerializer
from aparnik.utils.utils import is_app_installed
from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer
from aparnik.contrib.segments.api.serializers import BaseSegmentListPolymorphicSerializer
from ..models import Product, ProductPropertyMembership


# Product Details Serializer
class ProductPropertyMembershipListSerializer(ModelSerializer):
    title = serializers.SerializerMethodField()
    icon = serializers.SerializerMethodField()

    class Meta:
        model = ProductPropertyMembership
        fields = [
            'id',
            'title',
            'value',
            'icon',
        ]

    def get_title(self, obj):
        return obj.property.title

    def get_icon(self, obj):
        return FileFieldListSerailizer(obj.property.icon, many=False, read_only=True, context=self.context).data


class ProductListSerializer(BaseModelDetailSerializer):
    is_buy = serializers.SerializerMethodField()
    has_permit = serializers.SerializerMethodField()
    is_user_invited = serializers.SerializerMethodField()
    price_string = serializers.SerializerMethodField()
    is_discount_percent_expire_show = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(ProductListSerializer, self).__init__(*args, **kwargs)

        # if 'context' in kwargs:
        # self.fields['notification_type'] = NotificationTypeDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = Product
        fields = BaseModelDetailSerializer.Meta.fields + [
            'title',
            'price',
            'is_buy',
            'is_free',
            'is_user_invited',
            'has_permit',
            'price_fabric',
            'discount_percent',
            'delivery_type',
            'price_fabric_string',
            'price_string',


            'is_discount_percent_expire_show',
            'discount_percent_expire',
            'is_subscription',
        ]
        # TODO: fix read only fields
        read_only_fields = BaseModelDetailSerializer.Meta.read_only_fields + [
            'is_discount_percent_expire_show',
            'is_subscription',
            'is_buy',
            'has_permit',
            'is_user_invited',
            'price_string',
            'is_discount_percent_expire_show',
            'is_free'
        ]

    def get_is_buy(self, obj):
        return obj.is_buy(self.context['request'].user)

    # Is user eligible to use this product(purchased, free or shared product returns True)
    def get_has_permit(self, obj):
        user = self.context['request'].user
        return obj.has_permit(user=user)

    # Determines if a product shared with this user.
    def get_is_user_invited(self, obj):
        user = self.context['request'].user
        return obj.is_user_invited(user=user)

    def get_price_string(self, obj):
        user = self.context['request'].user
        return obj.price_string_description(user=user)

    def get_is_discount_percent_expire_show(self, obj):
        return obj.discount_percent > 0


# Product Details Serializer
class ProductDetailSerializer(ProductListSerializer):
    slider_segment = serializers.SerializerMethodField()
    properties = serializers.SerializerMethodField()
    url_products_sharing = serializers.SerializerMethodField()
    url_products_sharing_set = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(ProductDetailSerializer, self).__init__(*args, **kwargs)

        # if 'context' in kwargs:
        # self.fields['notification_type'] = NotificationTypeDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = Product
        fields = ProductListSerializer.Meta.fields + [
            'slider_segment',
            'properties',
            'further_details',
            'url_products_sharing',
            'url_products_sharing_set',
            'aparnik_bon_return',
            'aparnik_bon_return_value_string',
            'aparnik_bon_return_expire_date',
            'maximum_use_aparnik_bon',
            'has_permit_use_wallet',
        ]

        read_only_fields = ProductListSerializer.Meta.read_only_fields + ['slider_segment',
                                                                          'properties',
                                                                          'url_products_sharing',
                                                                          'url_products_sharing_set']

    def get_slider_segment(self, obj):
        return BaseSegmentListPolymorphicSerializer(obj.slider_segment_obj, many=False, read_only=True, context=self.context).data if obj.slider_segment_obj else None

    def get_properties(self, obj):
        return ProductPropertyMembershipListSerializer(obj.productpropertymembership_set.all(), many=True, read_only=True, context=self.context).data

    def get_url_products_sharing(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_products_sharing_uri())

    def get_url_products_sharing_set(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_product_sharing_add_uri())


# Product List Polymorphic
class ProductListPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):
        model_serializer_mapping = {
            Product: ProductListSerializer,
        }

        # Course
        if is_app_installed('aparnik.packages.educations.courses'):
            from aparnik.packages.educations.courses.models import Course
            from aparnik.packages.educations.courses.api.serializers import BaseCourseListPolymorphicSerializer
            model_serializer_mapping[Course] = BaseCourseListPolymorphicSerializer

        # File
        if is_app_installed('aparnik.packages.shops.files'):
            from aparnik.packages.shops.files.models import File
            from aparnik.packages.shops.files.api.serializers import FileListPolymorphicSerializer
            model_serializer_mapping[File] = FileListPolymorphicSerializer

        # Subscription
        if is_app_installed('aparnik.packages.shops.subscriptions'):
            from aparnik.packages.shops.subscriptions.models import Subscription
            from aparnik.packages.shops.subscriptions.api.serializers import SubscriptionListSerializer
            model_serializer_mapping[Subscription] = SubscriptionListSerializer

        self.model_serializer_mapping = model_serializer_mapping
        super(ProductListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


# Product Details Polymorphic
class ProductDetailsPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):
        model_serializer_mapping = {
            Product: ProductDetailSerializer,
        }

        # Course
        if is_app_installed('aparnik.packages.educations.courses'):
            from aparnik.packages.educations.courses.models import Course
            from aparnik.packages.educations.courses.api.serializers import BaseCourseDetailsPolymorphicSerializer
            model_serializer_mapping[Course] = BaseCourseDetailsPolymorphicSerializer

        # File
        if is_app_installed('aparnik.packages.shops.files'):
            from aparnik.packages.shops.files.models import File
            from aparnik.packages.shops.files.api.serializers import FileDetailsPolymorphicSerializer
            model_serializer_mapping[File] = FileDetailsPolymorphicSerializer

        # Subscription
        if is_app_installed('aparnik.packages.shops.subscriptions'):
            from aparnik.packages.shops.subscriptions.models import Subscription
            from aparnik.packages.shops.subscriptions.api.serializers import SubscriptionDetailSerializer
            model_serializer_mapping[Subscription] = SubscriptionDetailSerializer

        self.model_serializer_mapping = model_serializer_mapping
        super(ProductDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.packages.educations.progresses.models import Progresses, ProgressSummary
from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.contrib.users.admin import get_user_search_fields


# Register your models here.
class ProgressesAdmin(BaseModelAdmin):
    list_display = ['user_obj', 'file_obj', 'status', 'is_published']
    list_filter = ['status', 'created_at']
    search_fields = get_user_search_fields('user_obj') + ['file_obj__title']
    raw_id_fields = ['user_obj', 'file_obj']
    exclude = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = ProgressesAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Progresses


class ProgressesSummaryAdmin(BaseModelAdmin):
    list_display = ['user_obj', 'percentage', 'model']
    list_filter = []
    search_fields = get_user_search_fields('user_obj') + ['user_obj__title']
    raw_id_fields = ['user_obj', ]
    exclude = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = ProgressesSummaryAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = ProgressSummary


admin.site.register(Progresses, ProgressesAdmin)
admin.site.register(ProgressSummary, ProgressesSummaryAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-04-23 10:58


import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0017_auto_20190319_1744'),
    ]

    operations = [
        migrations.AlterField(
            model_name='product',
            name='aparnik_bon_return_value',
            field=models.IntegerField(default=-1, help_text='If the number is set to -1, then the value in the settings will apply', validators=[django.core.validators.MinValueValidator(-1), django.core.validators.MaxValueValidator(100)], verbose_name='Bon Return'),
        ),
        migrations.AlterField(
            model_name='product',
            name='maximum_use_aparnik_bon_value',
            field=models.IntegerField(default=-1, help_text='If the number is set to -1, then the value in the settings will apply, If the number is set to -2, then no limitation execution.', validators=[django.core.validators.MinValueValidator(-2), django.core.validators.MaxValueValidator(100)], verbose_name='Maximum use aparnik bon'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.http import Http404
from django.shortcuts import render, get_list_or_404, get_object_or_404, redirect
from django.urls import reverse

from aparnik.settings import aparnik_settings
from .models import Payment, PaymentBank
from azbankgateways import bankfactories, default_settings as bank_settings


# Create your views here.
def payment(request, uuid):
    payments_obj = get_list_or_404(Payment.objects.all(), uuid=uuid)
    pay_obj = payments_obj[0]

    if pay_obj.order_obj.status == Payment.STATUS_WAITING and pay_obj.status == Payment.STATUS_WAITING:

        if aparnik_settings.BANK_ACTIVE:
            order = pay_obj.order_obj
            amount = order.get_total_cost()
            user_mobile_number = order.user.username
            factory = bankfactories.BankFactory()
            bank = factory.create()
            bank.set_request(request)
            bank.set_amount(amount)
            # bank.set_client_callback_url(reverse('callback-gateway'))
            bank.set_client_callback_url(reverse('aparnik:shops:payments:call-back'))
            bank.set_mobile_number(user_mobile_number)
            bank_record = bank.ready()
            PaymentBank.objects.create(
                bank_record=bank_record,
                payment=pay_obj
            )
            return bank.redirect_gateway()
        else:
            pay_obj.success()

    context = {
        'obj': pay_obj,
        'app_url': request.build_absolute_uri(pay_obj.get_api_uri()),
    }

    if pay_obj.is_success():
        return render(request=request, template_name='suit/paysuc.html', status=200, context=context)

    return render(request=request, template_name='suit/payunsuc.html', status=400, context=context)


def callback_view(request):
    tracking_code = request.GET.get(bank_settings.TRACKING_CODE_QUERY_PARAM, None)
    if not tracking_code:
        raise Http404

    payment_bank = get_object_or_404(
        PaymentBank,
        bank_record__tracking_code=tracking_code
    )
    if payment_bank.bank_record.is_success:
        if payment_bank.payment.status == Payment.STATUS_WAITING:
            payment_bank.payment.success()
    else:
        payment_bank.payment.cancel()
    return redirect(request.build_absolute_uri(payment_bank.payment.get_url()))



from rest_framework import serializers

from aparnik.contrib.basemodels.api.admin.serializers import BaseModelAdminSerializer


from aparnik.packages.shops.products.models import Product


class ProductAdminListSerializer(BaseModelAdminSerializer):

    def __init__(self, *args, **kwargs):
        super(ProductAdminListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Product
        fields = BaseModelAdminSerializer.Meta.fields + [

            'title',

        ]

        read_only_fields = BaseModelAdminSerializer.Meta.read_only_fields + [

        ]


from django.conf.urls import url, include

from .views import AuditSortAdminAPIView, AuditAdminListAPIView

app_name = 'audits'

urlpatterns = [
    url(r'^$', AuditAdminListAPIView.as_view(), name='list'),
    url(r'^sort/$', AuditSortAdminAPIView.as_view(), name='sort'),
]



# Generated by Django 2.2 on 2021-09-15 11:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0003_remove_category_pages'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='is_show',
            field=models.BooleanField(default=True, verbose_name='Is Show'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.contrib.users.admin import get_user_search_fields
from .models import Voucher
# from questionsanswer.models import QA, Like


class VoucherAdmin(BaseModelAdmin):

    fields = ['user_obj', 'quantity', 'order_item_obj', 'expire_at', 'is_active', 'is_spent', 'quantity_remain']
    list_display = ['user_obj', 'quantity', 'is_spent', 'quantity_remain', 'expire_at', 'is_active', ]
    list_filter = []
    search_fields = get_user_search_fields('user_obj')
    exclude = []
    dynamic_raw_id_fields = []
    inlines = []
    raw_id_fields = ['user_obj', 'order_item_obj', ]

    def __init__(self, *args, **kwargs):
        Klass = VoucherAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields
        self.inlines = Klass_parent.inlines + self.inlines

    class Meta:
        model = Voucher


admin.site.register(Voucher, VoucherAdmin)
